=begin rdoc
Based on  CNC 2.5D Profile
			Pieces of this code came from the following:
			-Hatchfaces.rb by Joel Gustafsson 
			-TIG-Smart_offset.rb

			== Disclaimer
THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES,
INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
FOR A PARTICULAR PURPOSE.  PLEASE VERIFY GCODE CREATED PRIOR TO RUNNING ON CNC.  I AM NOT
RESPONSIBLE FOR DAMAGE AS A RESULT OF USING THIS SCRIPT.

Permission to use, copy, modify, and distribute this software for 
any purpose and without fee is hereby granted, provided that the above
copyright notice appear in all copies.

SU Version:: I developed this on SketchUp version 8. I have not tested on previous versions.
Description:: Creates gcode for Laser applications using 2D models in Sketchup
Usage:: Model must be in group.  Select group and activate plugin.  Answer questions and the plugin will create gcode.
Requires:: TIG-Smart_offset.rb
=end

Sketchup::require 'sketchup'
Sketchup.send_action "showRubyPanel:"
Sketchup::require 'TIG-Smart_offset.rb'

#################################################
module CNC
#################################################
class CNC_Profile
	
def activate	         # 0.3 Encapsulated variable declaration as well as sequential execution inside the activate method
	@height = 0
	@mill_diameter = nil
	@cut_feedrate = nil
	@plunge_feedrate = nil
	@retract_feedrate = nil
	@free_feedrate = nil
	@safety_height = nil
	@cut_depth = nil
	@file_out = []
	@mode = nil
	@igroup = nil
	@igroup_name = nil
	@gp = nil
	@cut_to_keep = nil
	@cut_direction = nil
	@laser_intensity = nil
	@stock2leave_inside = nil
	@stock2leave_outside = nil
	Sketchup.active_model.options["UnitsOptions"]["SuppressUnitsDisplay"]=true
	@units = Sketchup.active_model.options["UnitsOptions"]["LengthUnit"]
	Sketchup.active_model.options["UnitsOptions"]["LengthPrecision"]=3
	status = Sketchup.vcb_label = "Status"
	inputs
	create_layers
	find_bounds
	mode
	deactivate
	
end # def activate

#################################################

def deactivate
	Sketchup::set_status_text("",SB_PROMPT)
	Sketchup::set_status_text("",SB_VCB_LABEL)
	Sketchup::set_status_text("",SB_VCB_VALUE)
end

def exit
	@exit = true
	Sketchup.send_action "selectSelectionTool:"
end #exit

########################################  

	def inputs
		status = Sketchup.vcb_value = "Configuring Laser Job"
		dirpath = Sketchup.find_support_file "Plugins", ""
		dirpath = dirpath + "/CNC_Files"                     
		filename = '/CNC_Profile_Defaults.txt'               
		defaultfile = dirpath + filename
		puts(defaultfile)
			if File.file?(defaultfile)
				defaults=[]
				textfile = File.new(defaultfile, 'r' )
				textfile.readlines().each do |item|
				defaults.push (item[0..-2])
				end
				textfile.close()
			else
				defaults = [0.1, 200,500,128]   # MV Changed all to_f into to_l
			end #if File.file?()
	
		prompts = ['Beam Spot Size', 'Cutting Feedrate', 'Seek Feedrate ', 'Laser Intensity']
		
		options = ['','','','']
		input = UI.inputbox  prompts, defaults, options, "Laser Cutting Settings"
		if input
			textfile = File.new(defaultfile, "w" )
			input.each do | item |
				textfile.puts(item)
			end
		end
		textfile.close()	

		@mill_diameter = input[0].to_l / 2   
		@cut_depth = 0.1
		@cut_feedrate = input[1]
		@plunge_feedrate = 5000
		@retract_feedrate = 5000
		@free_feedrate = input[2]
		@safety_height = 0.1
		@stock2leave_inside = 0
		@stock2leave_outside = 0
		@laser_intensity = input[3]
		@mode = 0 # Single pass = 0, multipass = 1
		
		#if input[10] == "Inside"
		#	@cut_to_keep = 0
		#elsif input[10] == "Outside"
		#	@cut_to_keep = 1
		#elsif input[10] == "Both"
			@cut_to_keep = 2
		#end
		
		#if input[11] == "Climb"
		#	@cut_direction = 0
		#elsif input [11] == "Conventional"
			@cut_direction = 1
		#end
	end #inputs
#################################################
	def create_layers
	   #create seperate layers to display our router paths on.
		model = Sketchup.active_model
		layers = model.layers
		@path_layer = layers.add "path"
		@first_layer = model.active_layer = layers[0]

	end # create_layers
######################################################
	def find_bounds
		model = Sketchup.active_model
		selection = model.selection
		unless selection[0] && selection[0].is_a?(Sketchup::Group)		
			UI.messagebox "Selection is not a group!", MB_OK
			exit                                                        
		end
		@corner_lfb = model.bounds.corner(0) #left front bottom               
		@corner_rfb = model.bounds.corner(1) #right front bottom      
		@corner_lbb = model.bounds.corner(2) #left back bottom
		@corner_rbb = model.bounds.corner(3) #right back bottom
		@corner_lft = model.bounds.corner(4) #left front top
		@corner_rft = model.bounds.corner(5) #right front top
		@corner_lbt = model.bounds.corner(6) #left back top
		@corner_rbt = model.bounds.corner(7) #right back top
		thk = @corner_lft.z - @corner_lfb.z
		@cptx = (@corner_rfb.x-@corner_lfb.x)/2
		@cpty = (@corner_lbb.y-@corner_lfb.y)/2
		center = model.bounds.center
		@center_pt = Geom::Point3d.new center.x, center.y, 0
		if thk.to_l > @safety_height                    # 0.3 added to_l
			UI.messagebox "This plugin is designed to process 2D drawings", MB_OK
			exit            
		end
		@length = @corner_rfb.x-@corner_lfb.x #0.3 Added
		@width = @corner_lbb.y-@corner_lfb.y
		@thick = @corner_lft.z-@corner_lfb.z
		
	end # def find_bounds
######################################################
	def mode
		@outside_final_array = []
		@inside_final_array = []
		
		if @mode == 0 #Single step mode
			@height = 0.to_f
			puts (intersect)
			puts (@cut_to_keep)

			if @cut_to_keep == 0
				puts (inside_faces)
				puts (inside_offset)
				puts (erase_inside_group)
				puts (erase_outside_group)
			end
			if @cut_to_keep == 1
				puts (outside_faces)
				puts (outside_offset)
				puts (erase_inside_group)
				puts (erase_outside_group)
			end	
			if @cut_to_keep == 2
				puts (inside_faces)
				puts (outside_faces)
				puts (inside_offset)
				puts (outside_offset)
				puts (erase_inside_group)
				puts (erase_outside_group)
			end			
			puts (gcode_write)
					
		end #if
		if @mode == 1 #multipass mode
			@height = @corner_lft.z
			while @height > 0.to_f
				puts (intersect)
				puts (@cut_to_keep)
				if @cut_to_keep == 0 
					puts (inside_faces)
					puts (inside_offset)
					puts (erase_inside_group)
					puts (erase_outside_group)
				end
				if @cut_to_keep == 1 
					puts (outside_faces)
					puts (outside_offset)
					puts (erase_inside_group)
					puts (erase_outside_group)
				end
				if @cut_to_keep == 2
					puts (inside_faces)
					puts (outside_faces)
					puts (inside_offset)
					puts (outside_offset)
					puts (erase_inside_group)
					puts (erase_outside_group)
					
				end
				@height -= @cut_depth
			end #while @height > 0
			@height = 0.to_f
			puts (intersect)
			puts (@cut_to_keep)
			if @cut_to_keep == 0
				puts (inside_faces)
				puts (inside_offset)
				puts (erase_inside_group)
				puts (erase_outside_group)
			end
			if @cut_to_keep == 1
				puts (outside_faces)
				puts (outside_offset)
				puts (erase_inside_group)
				puts (erase_outside_group)

			end	
			if @cut_to_keep == 2
				puts (inside_faces)
				puts (outside_faces)
				puts (inside_offset)
				puts (outside_offset)
				puts (erase_inside_group)	
				puts (erase_outside_group)
			end			
			puts (gcode_write)

		end #if
	end #def mode
######################################################
	def intersect #intersect model with depth slice planes
		status = Sketchup.vcb_value = "Intersecting Model"
		model = Sketchup.active_model
		selection = model.selection
		entities = model.entities
		@igroup = entities.add_group
		@igroup.name = "inside_group"
		@igroup_name = @igroup.name
		ents = @igroup.entities

		new_face = ents.add_face @corner_lfb, @corner_lbb, @corner_rbb, @corner_rfb
		new_face.reverse!
			sc = Geom::Transformation.scaling @center_pt, 1.5
			tr = Geom::Transformation.translation [0, 0, @height]
			ents.transform_entities sc, new_face
			ents.transform_entities tr, new_face
				### The intersect_with method is used to intersect entities, a component instance,
				### or group with an entities object.
				###
				### entities.intersect_with recurse, transformation1, entities1, transformation2,
				### hidden, entities2
				###
				### recurse - true if you want this entities object to be recursed
				###   (intersection lines will be put inside of groups and components within this
				###   entities object)
				### transformation1 - the transformation for this entities object
				### entities1 - the entities (group etc) where you want the intersection
				###   lines to appear
				### transformation2 - the transformation for entities1
				### hidden - true if you want hidden geometry in this entities object to be used in
				###   the intersection  (it's false in section cuts)
				### entities2 - an entities object (or an array of entities ?)
			ents.intersect_with false, @igroup.transformation, @igroup,
				@igroup.transformation, false, selection
			ents.intersect_with false, @igroup.transformation, @igroup, 
				@igroup.transformation, false, selection 
				#need to run interest twice to get the holes to intersect with face and have separate
				#faces for the holes and the outer face.
				
			edges2go = []
			@igroup.entities.each {|e| 
				if e.class==Sketchup::Edge and not e.faces[1]
					edges2go << e
				end
			}
			@igroup.entities.erase_entities(edges2go) #erases the 4 original edges of new_face
	
			@igroup2 = @igroup.copy
			@igroup2.name = "outside_group"
			@igroup2_name = @igroup2.name
			
			edges3go = []
			@igroup2.entities.each {|e| 
				if e.class==Sketchup::Edge and e.faces[1]
					edges3go << e
				end
			}
			@igroup2.entities.erase_entities(edges3go)
		
	end #def intersect
#################################################
	def inside_faces
		status = Sketchup.vcb_value = "Creating Inside Faces"
		sel = Sketchup.active_model.entities
		model=Sketchup.active_model
		groups=[]
		model.definitions.each{|d|
			groups << d if d.group?
		}

		name="inside_group"
		group=nil
		groups.each{|g|
			group=g.instances[0] if g.instances[0].name==name
		}
		ents = group.entities
		@face = nil
		@edge = nil
		faces = []
		edges = []
		faces2go = []
		ents.each{|e|
			if e.class == Sketchup::Face
				faces << e
			end
			if e.class == Sketchup::Edge
				edges << e
			end
		}
		@face = faces[0] if faces
		faces.each{|face|
			face.loops.each{|loop|ents.add_face(loop.vertices)}   # make innerfaces and all faces
			@face = ents.add_face(face.outer_loop.vertices)         # make outer faces again to be sure
				ents.each{|face| 
					next if face.class!=Sketchup::Face     
						face.edges.each{|e|
						if e.faces[1]  #was if not
					break
				end
					faces2go << face
					}
				} 
		}
		ents.erase_entities(faces2go)   #erase all but inner faces 

	end 
#################################################

	def outside_faces
		status = Sketchup.vcb_value = "Creating Outside Faces"
		sel = Sketchup.active_model.entities
		model=Sketchup.active_model
		groups=[]
		model.definitions.each{|d|
			groups << d if d.group?
		}

		name="outside_group"
		group=nil
		groups.each{|g|
			group=g.instances[0] if g.instances[0].name==name
		}
		ents = group.entities
		@face = nil
		@edge = nil
		faces = []
		edges = []
		faces2go = []
		ents.each{|e|
			if e.class == Sketchup::Face
				faces << e
			end
			if e.class == Sketchup::Edge
				edges << e
			end
		}
		@face = faces[0] if faces
		faces.each{|face|
			face.loops.each{|loop|ents.add_face(loop.vertices)}   # make innerfaces and all faces
			@face = ents.add_face(face.outer_loop.vertices)         # make outer faces again to be sure
				ents.each{|face| 
					next if face.class!=Sketchup::Face     
						face.edges.each{|e|
						if not e.faces[1]  #was if not
					break
				end
					faces2go << face
					}
				} 
		}
		ents.erase_entities(faces2go)   #erase all but inner faces 

	end 
	
#################################################
	def outside_offset
		status = Sketchup.vcb_value = "Creating Offsets"
		model=Sketchup.active_model
		groups=[]
		model.definitions.each{|d|
			groups << d if d.group?
		}	
		name="outside_group"
		group=nil
		groups.each{|g|
			group=g.instances[0] if g.instances[0].name==name
		}
		faces = []
		vertex_array = []
		gent=group.entities
		gent.each{|e|
			if e.class==Sketchup::Face
			faces << e
		end
		}

		totalcount = faces.size

		faces.each do |face|

			face.loops.each{vertex_array = face.vertices}
				#find the closest point to origin
				point = Geom::Point3d.new 0,0,@height
				close_dist = nil
				close_point = nil
				vertex_array.each do |e|
					if point != e
						dist = point.distance e
						close_dist = dist if not close_dist
							if dist <= close_dist
								close_dist = dist
								close_point = e
							end
					end
				end
				#puts ("closest point: " +close_point.position.to_s)	#debugging
				#vertex_array.each {|pt| puts "Vertex Array: " + pt.position.to_s}	#debugging
				
				#here we rearrange the array "vertex_array" so that index 0 is the closest
				#point to origin

				@first = []
				@second = []
				@combined = []
				a = vertex_array.index(close_point)
				b = vertex_array.index(close_point)-1
				c = vertex_array.index(close_point)+1
				l = vertex_array.size
				#puts(a)	#debugging
				#puts(b)	#debugging
				#puts(c)	#debugging
				#puts(l)	#debugging
				if a == 0
					@combined = vertex_array 
				elsif a == l
					@first = vertex_array.pop
					@second = vertex_array.index(-1)
					@combined = @second | @first
				elsif a == 1
					@first = vertex_array.slice(a,l)
					@second = vertex_array - @first
					@combined = @first | @second
				else
					@first = vertex_array.slice(a,l)
					@second = vertex_array.slice(0,a)
					@combined = @first | @second
					
				end
				#first.each {|pt| puts "First: " + pt.position.to_s}	#debugging
				#second.each {|pt| puts "Second: " + pt.position.to_s}	#debugging
				#@combined.each {|pt| puts "Combined: " + pt.position.to_s}	#debugging
				@combined_ents = model.active_entities
				outside_face = @combined_ents.add_group
				outside_face.name = "outside_face"
				outside_face.material = "Green"
				ents = outside_face.entities
				face = ents.add_face @combined
					if @height > 0 
						face.reverse!
					end
				model.active_layer = @path_layer

				outside_offset = Sketchup.active_model.selection[0]; TIG::Smart_offset.new(face, (@mill_diameter.to_l+@stock2leave_outside.to_l))
				
				new_face=(face.outer_loop.edges[0].faces-[face])[0]
				vert = new_face.outer_loop.vertices
			
				#find the closest point to origin
				point = Geom::Point3d.new 0,0,@height
				close_dist = nil
				close_point = nil
				vert.each do |e|
					if point != e
						dist = point.distance e
						close_dist = dist if not close_dist
							if dist <= close_dist
								close_dist = dist
								close_point = e
							end
					end
				end
				
				#here we rearrange the array "vertex_array" so that index 0 is the closest
				#point to origin

				first = []
				second = []
				combined = []
				a = vert.index(close_point)
				b = vert.index(close_point)-1
				c = vert.index(close_point)+1
				l = vert.size
				#puts(a)	#debugging
				#puts(b)	#debugging
				#puts(c)	#debugging
				#puts(l)	#debugging
				if a == 0
					combined = vert 
				elsif a == l
					first = vert.pop
					second = vert.index(-1)
					combined = second | first
				elsif a == 1
					first = vert.slice(a,l)
					second = vert - first
					combined = first | second
				else
					first = vert.slice(a,l)
					second = vert.slice(0,a)
					combined = first | second
					
				end
				if @cut_direction == 1
					combined.reverse!
				end
				f = combined.first
				fp = f.position
				plunge = "G01 Z#{@height} F#{@plunge_feedrate}"
				retract = "G00 Z#{@safety_height} F#{@retract_feedrate} M05"	
				start = "G01 X#{fp.x} Y#{fp.y} Z#{fp.z} F#{@cut_feedrate} M06 S#{@laser_intensity}"	
				start1 = "G00 X#{fp.x} Y#{fp.y} F#{@free_feedrate}"
				
				
				@outside_final_array.push (start1)
				@outside_final_array.push (plunge)
				
				combined.each {|a|
					position = a.position
					write = "G01 X#{position.x} Y#{position.y} Z#{position.z} F#{@cut_feedrate} M06 S#{@laser_intensity}"
					@outside_final_array.push (write)
					}		
				@outside_final_array.push (start)
			
				if totalcount > (1) || @height == 0
					@outside_final_array.push (retract)	
				end

				puts (erase_outside_face)
			model.active_layer = @first_layer	
		end #faces.each do
		
	end #outside_offset
	
#################################################
	def inside_offset
		status = Sketchup.vcb_value = "Creating Offsets"
		model=Sketchup.active_model
		groups=[]
		model.definitions.each{|d|
			groups << d if d.group?
		}	
		name="inside_group"
		group=nil
		groups.each{|g|
			group=g.instances[0] if g.instances[0].name==name
		}
		faces = []
		vertex_array = []
		gent=group.entities
		gent.each{|e|
			if e.class==Sketchup::Face
			faces << e
		end
		}

		totalcount = faces.size
		
		faces.each do |face|
			
			face.loops.each{vertex_array = face.vertices}			
				#find the closest point to origin
				point = Geom::Point3d.new 0,0,@height
				close_dist = nil
				close_point = nil
				vertex_array.each do |e|
					if point != e
						dist = point.distance e
						close_dist = dist if not close_dist
							if dist <= close_dist
								close_dist = dist
								close_point = e
							end
					end
				end
				#puts ("closest point: " +close_point.position.to_s)	#debugging			
				first = []
				second = []
				combined = []
				a = vertex_array.index(close_point)
				b = vertex_array.index(close_point)-1
				c = vertex_array.index(close_point)+1
				l = vertex_array.size
				#puts(a)	#debugging
				#puts(b)	#debugging
				#puts(c)	#debugging
				#puts(l)	#debugging
				if a == 0
					combined = vertex_array 
				elsif a == l
					first = vertex_array.pop
					second = vertex_array.index(-1)
					combined = second | first
				elsif a == 1
					first = vertex_array.slice(a,l)
					second = vertex_array - first
					combined = first | second
				else
					first = vertex_array.slice(a,l)
					second = vertex_array.slice(0,a)
					combined = first | second
				end
				#first.each {|pt| puts "First: " + pt.position.to_s}	#debugging
				#second.each {|pt| puts "Second: " + pt.position.to_s}	#debugging
				#combined.each {|pt| puts "Combined: " + pt.position.to_s}	#debugging			
				combined_ents = model.active_entities
				inside_face = combined_ents.add_group
				inside_face.name = "inside_face"
				inside_face.material = "Green"
				ents = inside_face.entities
				face = ents.add_face combined
					if @height > 0 
						face.reverse!
					end
				model.active_layer = @path_layer
				inside_offset = Sketchup.active_model.selection[0]; TIG::Smart_offset.new(face, (-1*@mill_diameter.to_l+@stock2leave_inside.to_l))
				inner_loops=face.loops-[face.outer_loop]
				new_faces=[]
				inner_loops.each{|loop| new_faces << (loop.edges[0].faces-[face]) }
				new_faces.flatten!
				new_faces.uniq!
				
				vert = new_faces[0].vertices				

				#find the closest point to origin
				point = Geom::Point3d.new 0,0,@height
				close_dist = nil
				close_point = nil
				vert.each do |e|
					if point != e
						dist = point.distance e
						close_dist = dist if not close_dist
							if dist <= close_dist
								close_dist = dist
								close_point = e
							end
					end
				end	
				#here we rearrange the array "vertex_array" so that index 0 is the closest
				#point to origin
				first = []
				second = []
				combined = []
				a = vert.index(close_point)
				b = vert.index(close_point)-1
				c = vert.index(close_point)+1
				l = vert.size
				#puts(a)	#debugging
				#puts(b)	#debugging
				#puts(c)	#debugging
				#puts(l)	#debugging
				if a == 0
					combined = vert 
				elsif a == l
					first = vert.pop
					second = vert.index(-1)
					combined = second | first
				elsif a == 1
					first = vert.slice(a,l)
					second = vert - first
					combined = first | second
				else
					first = vert.slice(a,l)
					second = vert.slice(0,a)
					combined = first | second				
				end
				if @cut_direction == 0 #reverse cut direction for climb cutting
					combined.reverse!
				end
				f = combined.first
				fp = f.position			
		
				plunge = "G01 Z#{@height} F#{@plunge_feedrate} M03"
				retract = "G00 Z#{@safety_height} F#{@retract_feedrate} M04"	
				start = "G01 X#{fp.x} Y#{fp.y} Z#{fp.z} F#{@cut_feedrate} S#{@laser_intensity}"	
				start1 = "G00 X#{fp.x} Y#{fp.y} F#{@free_feedrate}"
				
				
				@inside_final_array.push (start1)
				@inside_final_array.push (plunge)
				
				combined.each {|a|
					position = a.position
					write = "G01 X#{position.x} Y#{position.y} Z#{position.z} F#{@cut_feedrate} M03 S#{@laser_intensity}"
					@inside_final_array.push (write)
					}		
				@inside_final_array.push (start)
			
				if totalcount > (1) || @height == 0
					@inside_final_array.push (retract)	
				end

				puts (erase_inside_face)
			model.active_layer = @first_layer	
		end #faces.each do
		
	end #inside_offset
	
#################################################
	def erase_outside_face

		model=Sketchup.active_model 
		groups=[] 
		model.definitions.each{|d| 
		  groups << d if d.group? 
		} 

		name="outside_face" 
		group=nil 
		groups.each{|g| 
		  group=g.instances[0] if g.instances[0].name==name 
		}
		edges2go = []
		faces2go = []
		group.entities.each {|e| 
			if e.class==Sketchup::Face
				faces2go << e
			end
			if e.class==Sketchup::Edge and e.faces[1]
				edges2go << e
			end
		}
		group.entities.erase_entities(faces2go)
		group.entities.erase_entities(edges2go)
		
	end #def erase_outside_face
#################################################
	def erase_outside_group
	
		model=Sketchup.active_model
		groups=[]
		model.definitions.each{|d|
			groups << d if d.group?
		}
		name="outside_group"
		group=nil
		groups.each{|g|
			group=g.instances[0] if g.instances[0].name==name
		}
		group.erase!
	end #def erase_outside_group
#################################################
	def erase_inside_face
	
		model=Sketchup.active_model
		groups=[]
		model.definitions.each{|d|
			groups << d if d.group?
		}
		name="inside_face"
		group=nil
		groups.each{|g|
			group=g.instances[0] if g.instances[0].name==name
		}
		edges2go = []
		faces2go = []
		group.entities.each {|e|
			if e.class==Sketchup::Face
				faces2go << e
			end		
			if e.class==Sketchup::Edge and not e.faces[1]
				edges2go << e
			end
		}
		group.entities.erase_entities(faces2go)
		group.entities.erase_entities(edges2go)
		
	end #def erase_inside_face
#################################################
	def erase_inside_group
	
		model=Sketchup.active_model
		groups=[]
		model.definitions.each{|d|
			groups << d if d.group?
		}
		name="inside_group"
		group=nil
		groups.each{|g|
			group=g.instances[0] if g.instances[0].name==name
		}
		group.erase!
	end #def erase_inside_group
#################################################
	def gcode_write
		status = Sketchup.vcb_value = "Writing G-Code"
		model = Sketchup.active_model
		filename = UI.savepanel "Save Gcode File", "c:\\", "default.nc"

#header
		outputfile = File.new( filename, "w" )
		outputfile.puts"( Lasercut Job  )" #0.3 added
		
		if @units == 2
			outputfile.puts"G21 ( Unit of measure: millimeter )"
		elsif @units == 0
			outputfile.puts"G20 ( Unit of measure: inch )"
		end
		outputfile.puts"G90 ( Absolute programming )"
		outputfile.puts"G30 ( Home Machine )"
				outputfile.puts""
		outputfile.puts"G00 Z#{@safety_height} F#{@retract_feedrate} (Move safety height)"
#inside
		outputfile.puts (@inside_final_array)
#outside
		outputfile.puts (@outside_final_array)
#footer
		outputfile.puts""
		outputfile.puts"G00 X0.00 Y.00 F#{@free_feedrate}( Move to origin )"
		outputfile.puts"M02 ( End of program )"
		outputfile.puts""
		outputfile.puts"end"
		outputfile.close()
	end #gcode_write
end #class Profile
#################################################
# Add a menu item to launch our plugin.

plugins_menu = UI.menu("Plugins")
 submenu = plugins_menu.add_submenu("OpenBuilds")
 submenu.add_item("Lasercut Profile") {Sketchup.active_model.select_tool CNC::CNC_Profile.new}
end #module